@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "gtkentrycompletion.h"
 #include "gtkentryprivate.h"
 #include "gtkcelllayout.h"
@@ -173,6 +174,19 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
   object_class->get_property = gtk_entry_completion_get_property;
   object_class->finalize = gtk_entry_completion_finalize;
 
+  /**
+   * GtkEntryCompletion::match-selected:
+   * @widget: the object which received the signal
+   * @model: the #GtkTreeModel containing the matches
+   * @iter: a #GtkTreeIter positioned at the selected match
+   * 
+   * The ::match-selected signal is emitted when a match from the list
+   * is selected. The default behaviour is to replace the contents of the
+   * entry with the contents of the text column in the row pointed to by
+   * @iter.
+   *
+   * Return value: %TRUE if the signal has been handled
+   */ 
   entry_completion_signals[MATCH_SELECTED] =
     g_signal_new ("match_selected",
                   G_TYPE_FROM_CLASS (klass),
@@ -183,6 +197,15 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
                   G_TYPE_BOOLEAN, 2,
                   GTK_TYPE_TREE_MODEL,
                   GTK_TYPE_TREE_ITER);
+		  
+  /**
+   * GtkEntryCompletion::action-activated:
+   * @widget: the object which received the signal
+   * @index: the index of the activated action
+   *
+   * The ::action-activated signal is emitted when an action
+   * is activated.
+   */
   entry_completion_signals[ACTION_ACTIVATED] =
     g_signal_new ("action_activated",
                   G_TYPE_FROM_CLASS (klass),
@@ -283,9 +306,6 @@ gtk_entry_completion_init (GtkEntryCompletion *completion)
   gtk_tree_selection_unselect_all (sel);
 
   cell = gtk_cell_renderer_text_new ();
-  g_object_set (cell, "cell_background_gdk",
-                &priv->tree_view->style->bg[GTK_STATE_NORMAL],
-                NULL);
   gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (priv->action_view),
                                               0, "",
                                               cell,
@@ -539,17 +559,16 @@ gtk_entry_completion_visible_func (GtkTreeModel *model,
   if (!completion->priv->case_normalized_key)
     return ret;
 
-  if (completion->priv->text_column >= 0)
-    ret = gtk_entry_completion_default_completion_func (completion,
-                                                        completion->priv->case_normalized_key,
-                                                        iter,
-                                                        NULL);
-
-  else if (completion->priv->match_func)
+  if (completion->priv->match_func)
     ret = (* completion->priv->match_func) (completion,
                                             completion->priv->case_normalized_key,
                                             iter,
                                             completion->priv->match_data);
+  else if (completion->priv->text_column >= 0)
+    ret = gtk_entry_completion_default_completion_func (completion,
+                                                        completion->priv->case_normalized_key,
+                                                        iter,
+                                                        NULL);
 
   return ret;
 }
@@ -608,9 +627,13 @@ gtk_entry_completion_list_button_press (GtkWidget      *widget,
                                &iter, path);
       gtk_tree_path_free (path);
 
+      g_signal_handler_block (completion->priv->entry,
+			      completion->priv->changed_id);
       g_signal_emit (completion, entry_completion_signals[MATCH_SELECTED],
                      0, GTK_TREE_MODEL (completion->priv->filter_model),
                      &iter, &entry_set);
+      g_signal_handler_unblock (completion->priv->entry,
+				completion->priv->changed_id);
 
       if (!entry_set)
         {
@@ -999,11 +1022,13 @@ gtk_entry_completion_delete_action (GtkEntryCompletion *completion,
  * @completion: A #GtkEntryCompletion.
  * @column: The column in the model of @completion to get strings from.
  *
- * Convience function for setting up the most used case of this code: a
+ * Convenience function for setting up the most used case of this code: a
  * completion list with just strings. This function will set up @completion
  * to have a list displaying all (and just) strings in the completion list,
  * and to get those strings from @column in the model of @completion.
  *
+ * This functions creates and adds a GtkCellRendererText for the selected column.
+
  * Since: 2.4
  */
 void
@@ -1116,10 +1141,20 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
   gint monitor_num;
   GdkRectangle monitor;
   GtkRequisition popup_req;
+  GtkTreeViewColumn *column;
+  GList *renderers;
 
   if (GTK_WIDGET_MAPPED (completion->priv->popup_window))
     return;
 
+  column = gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0);
+  renderers = gtk_tree_view_column_get_cell_renderers (column);
+  gtk_widget_ensure_style (completion->priv->tree_view);
+  g_object_set (GTK_CELL_RENDERER (renderers->data), "cell_background_gdk",
+                &completion->priv->tree_view->style->bg[GTK_STATE_NORMAL],
+                NULL);
+  g_list_free (renderers);
+
   gtk_widget_show_all (completion->priv->vbox);
 
   gdk_window_get_origin (completion->priv->entry->window, &x, &y);
